<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

// To support auto-updates, this needs to run during the wp_version_check cron job for privileged users.
$doing_cron = defined( 'DOING_CRON' ) && DOING_CRON;
if ( ! current_user_can( 'manage_options' ) && ! $doing_cron ) {
	return;
}
if (function_exists('p99_get_key')){
    if (trim(p99_get_key($item_name_99)) != "NOKEYFOUND") {
        $license_key_99 = trim(p99_get_key($item_name_99));
    } else {
        if (trim(p99_get_key('VIP Access Pass (365 Days)')) != "NOKEYFOUND") {
    		$license_key_99 = trim(p99_get_key('VIP Access Pass (365 Days)'));
        } else {
            if (trim(p99_get_key('VIP Access Pass (Lifetime)')) != "NOKEYFOUND") {
                $license_key_99 = trim(p99_get_key('VIP Access Pass (Lifetime)'));
            } else {
                $license_key_99 = "";
            }
        }
    }
}
